@@ -29,9 +29,12 @@
 // Properties
 // 	string Hostname, Username, Password
 //	int IPPort
+
+//#define DEBUGEFAX
+
 namespace gfax {
 	using Gtk;
-	using Mono.Posix;
+	using Mono.Unix;
 	using System;
 	using System.IO;
 	using System.Text;			//Encoding.ASCII.GetBytes
@@ -44,7 +47,7 @@ namespace gfax {
 
 	public class Efax
 	{	
-		private static Thread thread;
+		private static System.Threading.Thread thread;
 		string user;
 		
 		
@@ -64,6 +67,7 @@ namespace gfax {
 		
 		string[] modemType = {@"-j\Q4",@"-j\Q1",@"-j*F1",@"-j&H2&I0&R1&D3I4",@"-or"};
 		
+		
 		/*
 		# FCINIT='-j\Q4'                # AT&T (Dataport, Paradyne)
 		# FCINIT='-j\Q1'                # Motorola (Power Modem, 3400 Pro,...)
@@ -72,7 +76,8 @@ namespace gfax {
 		# FCINIT='-or'                  # Multi-Tech (for bit reversal)
 		*/
 		
-		TextWriter statusfile = null;
+		// Don't need this anymore
+		//TextWriter statusfile = null;
 		
 		public Efax () 
 		{
@@ -86,9 +91,8 @@ namespace gfax {
 			if ( Settings.EfaxModemReset == "" )
 				Settings.EfaxModemReset = "-kZ";
 				
-			statusfile = TextWriter.Synchronized(File.CreateText(gfax.Procfile));
-			//statusfile.WriteLine(Catalog.GetString("Starting efax transmit thread..."));
-			//statusfile.Flush();
+			//Don't need this anymore
+			//statusfile = TextWriter.Synchronized(File.CreateText(gfax.Procfile));
 		}
 		
 		public void close ()
@@ -220,13 +224,11 @@ namespace gfax {
 			pidInfo.Arguments = String.Concat(
 					"-q -sDEVICE=tiffg3 -r",
 					resolution, 
-					" -dNOPAUSE -dSAFER", 
+					" -dNOPAUSE -dSAFER -dBATCH", 
 					" -sOutputFile=",
 					dir_name,
 					"/tmp.%03d -sPAPERSIZE=",
-					papersize, " ",
-					fname,
-					"</dev/null >/dev/null");
+					papersize, " ",fname);
 			
 			#if DEBUGEFAX
 				Console.WriteLine("[Efax.send_init]\n  {0}", pidInfo.Arguments);
@@ -238,8 +240,6 @@ namespace gfax {
 			return dir_name;
 		}
 		
-		
-		
 		// Method send(string directory, contact)
 		//
 		// Sequence to send a file is:
@@ -283,7 +283,8 @@ namespace gfax {
 			string rand_file = rand.Next().ToString();
 			string tfname = String.Format("{0}/C_{1}.{2}",gfax.SpoolDirectory, contact.PhoneNumber, rand_file);
 			
-			// Open new status file
+			// TODO error stuff
+			//Open new status file
 			try { outfile = File.CreateText(tfname); }
 			catch (Exception e) { return; }
 			outfile.Write("Jobid=");
@@ -366,7 +367,7 @@ namespace gfax {
 		 */
 		private void efaxd (object state)
 		{
-			thread = Thread.CurrentThread;
+			thread = System.Threading.Thread.CurrentThread;
 			
 			StreamReader infile = null;
 			string[] sts = new string[11];
@@ -375,139 +376,179 @@ namespace gfax {
 			int retries;
 			string speakerModeVolume;
 			
-			statusfile.Write(" ");
-			
 			//string jobid, number, status, owner, pages, dials, error, sendat;
-			Thread.Sleep(30000); // Don't start immediately
-			statusfile.Flush();
-			statusfile.WriteLine(Catalog.GetString("Starting scan process"));
-			statusfile.WriteLine(Catalog.GetString("Scanning control files."));
-			statusfile.Flush();
+			System.Threading.Thread.Sleep(15000); // Don't start immediately
 			
 			while (true) {
-				string[] controlFiles = Directory.GetFiles(gfax.SpoolDirectory, "C_*");
-				//Console.WriteLine("Scanning control files.");
+				Application.Invoke (delegate {gfax.GAppbar.ClearStack();});
+				Application.Invoke (delegate {gfax.GAppbar.Push(Catalog.GetString("Scanning control files."));});
+
+				// while there are job files in the folder
+				while (Directory.GetFiles(gfax.SpoolDirectory, "C_*").Length > 0) {
 				
-				
-				foreach (string controlFile in controlFiles) {
-					statusfile.WriteLine(Catalog.GetString("Control file " + controlFile));
-					statusfile.Flush();
-					//fatalError = false;
-					try { 
-						infile = File.OpenText(controlFile); 
-						
-						for (int i=0; (buf = infile.ReadLine()) != null; i++)  {
-							string[] sa = buf.Split('=');
-							if ( sa[1].Length != 0 )
-								sts[i] = sa[1];
-							else
-								sts[i] = "-";
+					string[] controlFiles = Directory.GetFiles(gfax.SpoolDirectory, "C_*");
+					// for each job try send it					
+					foreach (string controlFile in controlFiles) {
+						#if DEBUGEFAX
+							Console.WriteLine("Control files are ----> {0}", controlFile);
+						#endif
+           
+						//Application.Invoke (delegate {gfax.GAppbar.ClearStack();});
+						Application.Invoke (delegate {gfax.GStatusTextBuffer.InsertAtCursor(
+								Catalog.GetString("Running job " + controlFile));});
+           
+						try { 
+							infile = File.OpenText(controlFile); 
+							
+							for (int i=0; (buf = infile.ReadLine()) != null; i++)  {
+								string[] sa = buf.Split('=');
+								if ( sa[1].Length != 0 )
+									sts[i] = sa[1];
+								else
+									sts[i] = "-";
+							}
+							infile.Close();
+							#if DEBUGEFAX
+								Console.WriteLine("[Efaxd] Buffer is {0}", buf);
+							#endif
 						}
-						infile.Close();
-					}
-					catch (Exception e) {continue; }
-					
-					//lines[num_lines++] = String.Format("{0}={1}={2}={3}={4}={5}={6}={7}\n",
-					//		sts[0],sts[2],sts[3],sts[4],sts[5],sts[6],sts[10],sts[7]);
-
-					// If we aborted for some reason check the retries
-					if (Convert.ToInt32(sts[6]) >= Settings.EfaxRetries) {
-						sts[10] = Catalog.GetString("Busy retries exceeded");
-						update_status_code(sts, controlFile, "F");
-						continue;
-					}
-					
-					sts[10] = "";	// clear error message
-					update_status_code(sts, controlFile, "R");
-					
-					// default "-iM1L0"
-					speakerModeVolume = String.Concat("-i", 
-							speakerMode[Settings.EfaxModemSpeakerMode],
-							speakerVolume[Settings.EfaxModemSpeakerVolume]);
-					
-					ProcessStartInfo pidInfo = new ProcessStartInfo();
-					pidInfo.FileName = "efax";
-					//pidInfo.FileName = "echo";					
-					
-					pidInfo.Arguments = String.Concat(
-						"-d /dev/", Settings.EfaxModemDevice,	//modem port
-						" -x ", Settings.EfaxLockfile, Settings.EfaxModemDevice,	// lockfile 
-						" \"",  Settings.EfaxModemInit, "\"",		// init sequence
-						" \"",  speakerModeVolume, "\"",	// speaker enable / mode
-						" -l ", Settings.FaxNumber,				// Our fax number
-						" \"",  Settings.EfaxModemReset, "\"",	// how to reset modem
-						" -h ", Settings.FaxNumber,			// should be name number and date
-						" -t ", sts[2],		// should make sure there are no bad chars
-						" -v i",
-						" ",    sts[1], "/tmp.*");		// files to send
-				    
-					#if DEBUGEFAX
-						Console.WriteLine("[Efax.send]\n  {0}", pidInfo.Arguments);
-					#endif
+						catch (Exception e) {continue; }
 						
-					Process pid = Process.Start(pidInfo);
-					pid.WaitForExit();
+						//lines[num_lines++] = String.Format("{0}={1}={2}={3}={4}={5}={6}={7}\n",
+						//		sts[0],sts[2],sts[3],sts[4],sts[5],sts[6],sts[10],sts[7]);
+           
+						// If we aborted for some reason check the retries
+						if (Convert.ToInt32(sts[6]) >= Settings.EfaxRetries) {
+							sts[10] = Catalog.GetString("Busy retries exceeded");
+							update_status_code(sts, controlFile, "F");
+							continue;
+						}
 						
-					#if DEBUGEFAX
-						Console.WriteLine("[Efax.efaxd] Exit code - {0}", pid.ExitCode);
-					#endif
+						sts[10] = "";	// clear error message
+						update_status_code(sts, controlFile, "R");
 						
-					switch (pid.ExitCode) {
-						case 0:
-							// If successful then mv the control file to the done queue and flag 
-							// a date for it's removal.
-							sts[10] = Catalog.GetString("Success");
-							update_status_code(sts, controlFile, "D");
-							// remove directory path
-							string basefilename = controlFile.Remove(0, gfax.SpoolDirectory.Length + 1);
-							string newfilename = String.Concat(gfax.SpoolDirectory,"/doneq/", basefilename);
-							File.Move(controlFile, newfilename);
-							Directory.Delete(sts[1], true);
-							break;
-						case 1:
-							// busy number, continue after timeout
-							// update listview as well
-							retries = Convert.ToInt32(sts[6]);
-							if (retries++ < Settings.EfaxRetries) {
-								sts[6] = retries.ToString();
-								update_status_code(sts, controlFile, "W");
-							}
-							else {
-								sts[10] = Catalog.GetString("Busy retries exceeded");
+						// default "-iM1L0"
+						speakerModeVolume = String.Concat("-i", 
+								speakerMode[Settings.EfaxModemSpeakerMode],
+								speakerVolume[Settings.EfaxModemSpeakerVolume]);
+						
+						string [] tifFiles = Directory.GetFiles(sts[1], "tmp.*");
+						StringBuilder filesToSend = new StringBuilder();
+						foreach (string f in tifFiles) {
+							filesToSend.Append(f);
+							filesToSend.Append(" ");
+						}
+						
+											
+						ProcessStartInfo pidInfo = new ProcessStartInfo();
+						pidInfo.FileName = "efax";
+						pidInfo.RedirectStandardError = true;
+						pidInfo.UseShellExecute = false;
+						//used for testing
+						//Thread.Sleep(5000);
+						//pidInfo.FileName = "echo";
+						//pidInfo.FileName = "sleep 5";										
+						
+						pidInfo.Arguments = String.Concat(
+							"-d /dev/", Settings.EfaxModemDevice,	//modem port
+							" -x ", Settings.EfaxLockfile, Settings.EfaxModemDevice,	// lockfile 
+							" \"",  Settings.EfaxModemInit, "\"",		// init sequence
+							" \"",  speakerModeVolume, "\"",	// speaker enable / mode
+							" -l \"", Settings.FaxNumber, "\"",				// Our fax number
+							" \"",  Settings.EfaxModemReset, "\"",	// how to reset modem
+							" -f /usr/bin/efaxfont",
+							" -h \'%d/%d\'", // should be name number and date
+							" -v i",			// session progress information
+							" -t T", sts[2],		// should make sure there are no bad chars
+							" ", filesToSend);		// files to send
+					    
+						#if sDEBUGEFAX
+							Console.WriteLine("[Efax.send]\n  {0}", pidInfo.Arguments);
+						#endif
+						Application.Invoke (delegate {gfax.GAppbar.ClearStack();});
+						Application.Invoke (delegate {gfax.GAppbar.Push(Catalog.GetString("Sending facsimile..."));});	
+						Process pid = Process.Start(pidInfo);
+						StreamReader myStreamReader = pid.StandardError;
+						
+						//start the pulser
+						Application.Invoke (delegate {gfax.Pulser.StartPulse();});
+						Application.Invoke (delegate {gfax.Status.Append("");});	
+						
+						while (!pid.HasExited) {
+							string str = myStreamReader.ReadLine();
+							Application.Invoke (delegate {gfax.Status.Append(str);});	
+							System.Threading.Thread.Sleep(200);
+						}
+						//stop the pulser.
+						Application.Invoke (delegate {gfax.Pulser.EndPulse();});
+																		
+						#if DEBUGEFAX
+							Console.WriteLine("[Efax.efaxd] Exit code - {0}", pid.ExitCode);
+						#endif
+							
+						switch (pid.ExitCode) {
+							case 0:
+								// If successful then mv the control file to the done queue and flag 
+								// a date for it's removal.
+								sts[10] = Catalog.GetString("Success");
+								update_status_code(sts, controlFile, "D");
+								// remove directory path
+								string basefilename = controlFile.Remove(0, gfax.SpoolDirectory.Length + 1);
+								string newfilename = String.Concat(gfax.SpoolDirectory,"/doneq/", basefilename);
+								File.Move(controlFile, newfilename);
+								break;
+							case 1:
+								// busy number, continue after timeout
+								// update listview as well
+								retries = Convert.ToInt32(sts[6]);
+								if (retries++ < Settings.EfaxRetries) {
+									sts[6] = retries.ToString();
+									update_status_code(sts, controlFile, "W");
+								}
+								else {
+									sts[10] = Catalog.GetString("Busy retries exceeded");
+									update_status_code(sts, controlFile, "F");
+								}
+								break;
+							case 2:
+								// fatal errors - no retry
+								//fatalError = true;
+								// change code in file
+								sts[10] = Catalog.GetString("Fatal error");
 								update_status_code(sts, controlFile, "F");
-							}
-								
-							break;
-						case 2:
-							// fatal errors - no retry
-							//fatalError = true;
-							// change code in file
-							sts[10] = Catalog.GetString("Fatal error");
-							update_status_code(sts, controlFile, "F");
-							break;
-						case 3:
-							// Modem error - no retry
-							//fatalError = true;
-							sts[10] = Catalog.GetString("Fatal modem error");
-							update_status_code(sts, controlFile, "F");
-							break;
-						case 4:
-							// Modem not responding
-							//fatalError = true;
-							sts[10] = Catalog.GetString("Modem not responding");
-							update_status_code(sts, controlFile, "B");
-							break;
-						case 5:
-							// Program terminated
-							//fatalError = true;
-							sts[10] = Catalog.GetString("Program terminated");
-							update_status_code(sts, controlFile, "F");
-							break;
-					}// end of switch
-
-					
-				} // end of foreach
-				Thread.Sleep(60000); // sleep 60 seconds	
+								break;
+							case 3:
+								// Modem error - no retry
+								//fatalError = true;
+								sts[10] = Catalog.GetString("Fatal modem error");
+								update_status_code(sts, controlFile, "F");
+								break;
+							case 4:
+								// Modem not responding
+								//fatalError = true;
+								sts[10] = Catalog.GetString("Modem not responding");
+								update_status_code(sts, controlFile, "B");
+								break;
+							case 5:
+								// Program terminated
+								//fatalError = true;
+								sts[10] = Catalog.GetString("Program terminated");
+								update_status_code(sts, controlFile, "F");
+								break;
+						}// end of switch
+						
+						//sleep between each fax for the modem to stablize
+						System.Threading.Thread.Sleep(3000); 
+					} // end of foreach
+					// This basically sleeps between busy numbers.
+					System.Threading.Thread.Sleep(30000); 
+				}
+				System.Threading.Thread.Sleep(30000); // sleep 30 seconds
+				// Moved this from case 0 above
+				#if DEBUGEFAX
+					Console.WriteLine("Deleting --->{0}", sts[1]);
+				#endif
+				//Directory.Delete(sts[1], true);
 			}// end of while
 		}
 		
