@@ -15,15 +15,17 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
+//#define DEBUGHYLAFAX
+
+
 namespace gfax {
-	using Mono.Posix;
+	using Mono.Unix;
 	using System;
 	using System.Collections;
 	
 	public class Fax
 	{
 		static Hylafax hfax;
-		static Mgetty mfax;
 		static Efax efax;
 		static bool firstRun = true;
 
@@ -41,38 +43,32 @@ namespace gfax {
 		
 		public class FaxRecQueue
 		{
-			public string Permissions;
 			public string Pages;
 			public string Status;			
-			public string Owner;			
 			public string Sender;
 			public string TimeReceived;			
 			public string Filename;			
 		}
 		
-		public static string get_server_status ()
+		public static string async_get_server_status ()
 		{
-			string status;
-			
+						
 			if (Settings.TransmitAgent == "hylafax") {
 				hfax = new Hylafax();
 				
 				if ( !hfax.connect() )
 					return (Catalog.GetString("No Connection"));
-				status = hfax.status("status");
+				hfax.asyncstatus("status");
 				hfax.close();
-				return (status);
-			}
-			
-			if (Settings.TransmitAgent == "mgetty") {
-					return ("Using Mgetty");
+				hfax = null;
+				return ("");
 			}
 			
 			if (Settings.TransmitAgent == "efax") {
 				//TODO put modem ready status here
 				if (firstRun) {
 					firstRun = false;					
-					return (Catalog.GetString("Efax transmit process running...\nScanning job files every 60 seconds."));
+					return (Catalog.GetString("Efax transmit process running...\nScanning job files every 30 seconds."));
 				}
 				else
 					return (null);
@@ -81,45 +77,47 @@ namespace gfax {
 			return (Catalog.GetString("Error transport agent not specified!"));
 		}
 
-		public static ArrayList get_queue_status (string queue)
+		public static ArrayList async_get_queue_status (string queue)
 		{
-			string reply = "";
-			
 			if (Settings.TransmitAgent == "hylafax") {
 				hfax = new Hylafax();
 				
-				// this is here just to make sure the headers in the listview appear
 				if ( hfax.connect() )
-					reply = hfax.status(queue);
+					hfax.asyncstatus(queue);
 				hfax.close();
+				hfax = null;
 			}		
 			
-			//if (Settings.TransmitAgent == "mgetty") {
-			//	list.Add("Using Mgetty");
-			//}
-			
 			if (Settings.TransmitAgent == "efax") {
-				reply = gfax.efax.status(queue);
+				string reply = gfax.efax.status(queue);
+				
+				if (queue == "doneq" || queue == "sendq")
+					return (parse_senddone(reply));
+				else // (queue == "recvq")
+					return (parse_receive(reply));
 			}
-			
-			if (queue == "doneq" || queue == "sendq")
-				return parse_senddone(reply);
-			else // (queue == "recvq")
-				return parse_receive(reply);
-			
+			return (null);
 		}
-		
+
 		// parse the send or done queue
 		public static ArrayList parse_senddone (string reply)
 		{
 			ArrayList list = new ArrayList();
-		
+			
 			if (reply.Length > 0) {
 				foreach (string s in reply.Split('\n')) {
 					FaxQueue hq = new FaxQueue();
-					if (s.Length == 0 )
-						break;
+					//Console.WriteLine("s length equals --> {0}", s.Length);				
+					if (s.Length == 0  )
+						break; 
+    	    		
     	    		string[] sa = s.Split('=');
+					
+					// TODO put error here about bad network comms
+					if (sa.Length != 8) {
+						break;
+					}
+					
 					if ( sa[0].Length != 0 )
 						hq.Jobid = sa[0];
 					else
@@ -212,51 +210,50 @@ namespace gfax {
 
 		// parse the receive queue
 		//
-		//Protect Page S  Owner        Sender/TSI  Recvd@ Filename
-		//-rw-r--    1    10       +49 30 7865224 10Nov04 fax00004.tif
+		//Page S Sender/TSI  Recvd@ Filename
+		//1 +49 30 7865224 10Nov04 fax00004.tif
 		// should come back as
-		//-rw-r--=1= =10=+49 30 7865224=10Nov04=fax00004.tif
+		//1= =+49 30 7865224=10Nov04=fax00004.tif
 		public static ArrayList parse_receive (string reply)
 		{
 			ArrayList list = new ArrayList();
 			
+						
 			if (reply.Length > 0) {
 				foreach (string s in reply.Split('\n')) {
+					
 					FaxRecQueue hq = new FaxRecQueue();
 					if (s.Length == 0 )
 						break;
+						
     	    		string[] sa = s.Split('=');
+    	    		if (sa.Length != 5) {
+						break;
+					}
+					
 					if ( sa[0].Length != 0 )
-						hq.Permissions = sa[0];
-					else
-						hq.Permissions = "";
-					if ( sa[1].Length != 0 )
-						hq.Pages = sa[1];
+						hq.Pages = sa[0];
 					else
 						hq.Pages = "";	
-					if ( sa[2].Length != 0 )
-						switch (sa[2]) {
-							case "*":
+					if ( sa[1].Length != 0 )
+						switch (sa[1]) {
+							case "N*":
 									hq.Status = Catalog.GetString("Receiving");
 									break;
-							case " ":
+							case "N ":
 							        hq.Status = Catalog.GetString("Done");
 							        break;
 						}
-					if ( sa[3].Length != 0 )
-						hq.Owner = sa[3];
-					else
-						hq.Owner = "";	
-					if ( sa[4].Length != 0 )
-						hq.Sender = sa[4];
+					if ( sa[2].Length != 0 )
+						hq.Sender = sa[2];
 					else
 						hq.Sender = "";
-					if ( sa[5].Length != 0 )
-						hq.TimeReceived = sa[5];
+					if ( sa[3].Length != 0 )
+						hq.TimeReceived = sa[3];
 					else
 						hq.TimeReceived = "";	
-					if ( sa[6].Length != 0 )
-						hq.Filename = sa[6];
+					if ( sa[4].Length != 0 )
+						hq.Filename = sa[4].Trim();
 					else
 						hq.Filename = "";
 					
@@ -266,10 +263,8 @@ namespace gfax {
 			} else  {		// no status
 			
 				FaxRecQueue hq = new FaxRecQueue();
-				hq.Permissions = "";
 				hq.Pages = "";
 				hq.Status = "";
-				hq.Owner = "";
 				hq.Sender = "";
 				hq.TimeReceived = "";
 				hq.Filename = "";
@@ -285,12 +280,14 @@ namespace gfax {
 		public static void sendfax (string fname)
 		{
 			string remote_fname;
-				
+			
+			Console.WriteLine("[Fax.sendfax] File name is : {0}", fname);
+			
 			if (Settings.TransmitAgent == "hylafax") {
-				hfax = new Hylafax();
+				Hylafax hfaxsf = new Hylafax();
 				// hylafax actually stores the file to the server
-				hfax.connect();
-				remote_fname = hfax.send_init(fname);
+				hfaxsf.connect();
+				remote_fname = hfaxsf.send_init(fname);
 				
 				// if "Cancel" button pressed on progess bar
 				if (remote_fname == "cancelled") 
@@ -300,37 +297,41 @@ namespace gfax {
 					Console.WriteLine("[Fax.sendfax] Remote file name is : {0}", remote_fname);
 				#endif
 				
-				IEnumerator enu = gfax.Destinations.GetEnumerator();
-    	  		while ( enu.MoveNext() ) {
-    	     		// TODO try catch exception here
-					
-					hfax.send(remote_fname, (GfaxContact)enu.Current);
+				//System.Threading.Thread.Sleep(2000);
 				
+				IEnumerator enu = gfax.Destinations.GetEnumerator();
+				
+				#if DEBUGHYLAFAX
+					Console.WriteLine("[Fax.sendfax] Destinations has a count of : {0}", gfax.Destinations.Count);
+				#endif
+    	  		
+				while ( enu.MoveNext() ) {
+    	     		// TODO try catch exception here
+					GfaxContact c = (GfaxContact)enu.Current;
+					
+					#if DEBUGHYLAFAX
+						Console.WriteLine("[Fax.sendfax] In while loop contact is -----> {0}", c.PhoneNumber);
+					#endif
+					
+					hfaxsf.send(remote_fname, c);
+					
+					#if DEBUGHYLAFAX
+						Console.WriteLine("[Fax.sendfax] In while loop bottom... ");
+					#endif
+					
+					//System.Threading.Thread.Sleep(1000);
 					// open the log and log out going fax to server
 					// Date Time PhoneNumber Organization ContactPerson
 					// log file is in ~/.etc/gfax
-					if (Settings.LogEnabled)
-						log_it((GfaxContact)enu.Current);
+					//if (Settings.LogEnabled)
+						//log_it((GfaxContact)enu.Current);
 				}
+				#if DEBUGHYLAFAX
+					Console.WriteLine("[Fax.sendfax] End of send contact loop...");
+				#endif
+				hfaxsf = null;
 			} 
 			
-			// Mgetty support
-			if (Settings.TransmitAgent == "mgetty") {
-				mfax = new Mgetty();
-				mfax.connect();
-				
-				remote_fname = mfax.send_init(fname);
-				if (remote_fname == "cancelled") 
-					return;
-				
-				IEnumerator enu = gfax.Destinations.GetEnumerator();
-    	  		while ( enu.MoveNext() ) {
-					mfax.send(remote_fname, (GfaxContact)enu.Current);
-					if (Settings.LogEnabled)
-						log_it((GfaxContact)enu.Current);
-				}
-
-			}
 			
 			//Efax transport
 			if (Settings.TransmitAgent == "efax") {
@@ -348,7 +349,27 @@ namespace gfax {
 				}
 			}
 		}
-		
+
+		public static bool recvfax (string fname)
+		{
+							
+			// Hylafax support
+			if (Settings.TransmitAgent == "hylafax") {
+				hfax = new Hylafax();
+				// hylafax actually stores the file to the server
+				hfax.connect();
+				return(hfax.getfile(fname));
+			} 
+
+			// Efax transport
+			if (Settings.TransmitAgent == "efax") {
+				return false;
+			}
+			
+			return false;
+		}
+
+
 		public static void log_it (GfaxContact contact)
 		{
 		
@@ -363,11 +384,13 @@ namespace gfax {
 				hfax.job_select(jobid);
 				hfax.job_suspend();
 				hfax.job_param_set("SENDTIME", sendat);
+				hfax.job_param_set("EXTERNAL", number);				
 				hfax.job_param_set("DIALSTRING", number);
 				hfax.job_param_set("MAXDIALS", dials);
 				hfax.job_submit();
+				hfax = null;
 			}
-			
+
 			/*
 			if (Settings.TransmitAgent == "efax") {
 				gfax.efax.job_kill(jobid);
@@ -376,6 +399,30 @@ namespace gfax {
 			*/
 
 		}
+		
+		public static void resubmit_job (string jobid, string number, string sendat, string dials)
+		{
+			// will have to query the job, get the file name
+			// possibly retrieve the file and then re-submit the job
+			
+			if (Settings.TransmitAgent == "hylafax") {
+				hfax = new Hylafax();
+				hfax.connect();
+				hfax.job_select(jobid);
+				
+				/*
+				hfax.job_reset("P");
+				hfax.job_suspend();
+				hfax.job_param_set("SENDTIME", sendat);
+				hfax.job_param_set("EXTERNAL", number);				
+				hfax.job_param_set("DIALSTRING", number);
+				hfax.job_param_set("MAXDIALS", dials);
+				hfax.job_submit();
+				hfax = null;
+				*/
+			}
+			
+		}
 
 
 		public static void delete_job (string jobid)
@@ -386,10 +433,7 @@ namespace gfax {
 				hfax.job_select(jobid);
 				hfax.job_kill();
 				hfax.job_delete();
-			}
-			
-			if (Settings.TransmitAgent == "mgetty") {
-				mfax = new Mgetty();
+				hfax = null;
 			}
 			
 			if (Settings.TransmitAgent == "efax") {
@@ -398,6 +442,21 @@ namespace gfax {
 			}
 
 		}
+		
+		public static int delete_file (string fname)
+		{
+			if (Settings.TransmitAgent == "hylafax") {
+				hfax = new Hylafax();
+				hfax.connect();
+				int reply = hfax.deletefile(fname);
+				hfax = null;
+				return (reply);
+			}
+			
+			if (Settings.TransmitAgent == "efax") {
+				return(0);
+			}
+			return(0);
+		}
 	}
-	
 }
