@@ -29,22 +29,40 @@
 // Properties
 // 	string Hostname, Username, Password
 //	int IPPort
+
+//#define DEBUGHYLAFAX
+
 namespace gfax {
-	using Mono.Posix;
+	using Mono.Unix;
 	using System;
 	using System.IO;
 	using System.Text;			//Encoding.ASCII.GetBytes
-	using System.Collections;	
+	using System.Collections;
+	using System.Threading;
 	using System.Net;
 	using System.Net.Sockets;
 
+	
 	public class Hylafax
 	{	
 		TcpClient mainclient = new TcpClient();
 		NetworkStream mainstream;
 		string host, user, pass;
 		int port;
+		bool permission;
 		
+		
+		public class StateObject
+		{
+			public TcpClient client = null;
+			public int totalBytesRead = 0;
+			public const int BufferSize = 256;
+			public string readType = null;
+			public byte[] buffer = new byte[BufferSize];
+			public StringBuilder messageBuffer = new StringBuilder();
+		}
+
+
 		// connect
 		//		return false on sucess or true on cancel
 		public bool connect ()
@@ -155,6 +173,7 @@ namespace gfax {
 		//	'status', 'sendq', 'doneq' or 'recvq'
 		//
 		// Return a list containing lines	
+/*		
 		public string status (string queue)
 		{
 			#if DEBUGHYLAFAX
@@ -162,9 +181,14 @@ namespace gfax {
 			#endif
 			// try 
 			return getfolder(queue);
-			
-			//string[] s = String.split(getfolder(queue), '\015\012');
-						
+		}
+*/
+		public void asyncstatus (string queue)
+		{
+			#if DEBUGHYLAFAX
+				Console.WriteLine ("[hylafax.status] queue is : {0}", queue);
+			#endif
+			asyncgetfolder(queue);
 		}
 
 		//	send_init (string filename)
@@ -262,7 +286,15 @@ namespace gfax {
 		//213 SENDTIME set to 20040310050900.
 		public void send (string remote_fname, GfaxContact contact)
 		{
+			#if DEBUGHYLAFAX
+				Console.WriteLine("Hylafax.send] top of method...");
+			#endif
+
 			string emailAddress = Settings.EmailAddress;
+			#if DEBUGHYLAFAX
+				Console.WriteLine("Hylafax.send] email address {0}", emailAddress);
+			#endif
+
 			string resolution = "98";
 			string emailNotify = "none";
 			
@@ -318,26 +350,119 @@ namespace gfax {
 			job_submit();
 		}
 		
+		public bool getfile (string fname)
+		{
+			G_ProgressBar pbar;
+			TcpClient myclient;
+			NetworkStream mystream;
+			FileStream fp;
+			string data;
+			double fsize = 0;
+			double totalBytes = 0;
+			double percent = 0;
+
+			// get the file size for the receive progress bar
+			write(mainstream, String.Concat("SIZE recvq/", fname, "\n"));
+			data = read(mainstream, mainclient);
+			try {
+				string[] s = data.Split();
+				fsize = Convert.ToDouble(s[1]);
+			} catch (Exception e) {
+				Console.WriteLine("Couldn't get file size");
+				Console.WriteLine("Hylafax exception {0}", e);
+				return (false);
+			}
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.getfile] \"File size is : {0}", fsize);
+			#endif
+			
+			// assume we have permission to open file
+			permission = true;
+			// PASV, RETR
+			myclient = recvfile_open(fname);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[Hylafax.getfile] Got tcpclient");
+			#endif
+			
+			mystream = myclient.GetStream();
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[Hylafax.getfile] Stream is open");
+			#endif
+			
+			if (!permission) {
+				return (false);
+			}
+			
+			try {
+				fp = new FileStream(String.Concat(gfax.SpoolDirectory,"/tif/",fname), FileMode.Create);
+			} catch (Exception e) {
+				Console.WriteLine(e.ToString());
+				return(false);
+			}
+			
+			byte[] bytes = new byte[256];
+			
+			do {
+				try {
+					gfax.MainProgressBar.Fraction = percent;
+					while (Gtk.Application.EventsPending ())
+                		Gtk.Application.RunIteration ();
+						
+					int len = mystream.Read(bytes, 0, bytes.Length);
+					if (len > 0 ) {
+						totalBytes = totalBytes + len;
+						percent = totalBytes / fsize;
+						if (percent <= 1.0)
+							gfax.MainProgressBar.Fraction = percent;
+						//GLib.MainContext.Iteration ();
+						while (Gtk.Application.EventsPending ())
+                			Gtk.Application.RunIteration ();
+						fp.Write(bytes,0,len);
+					} 
+				} catch ( Exception e ) {
+					Console.WriteLine(e.ToString());
+					return(false);
+				}
+				// Seems if we don't sleep here for a bit, we never read
+				// all the data???  Maybe a Hylafax thing.
+				System.Threading.Thread.Sleep(1);
+					
+			} while (mystream.DataAvailable);
+			
+			fp.Close();
+			recvfile_close(myclient);
+			
+			// valid file received
+			return(true);
+		}
+
+			
+
 		private string job_new ()
 		{
 			string data;
 			string[] tmp;
 			//string jobid;
 			
-			write(mainstream, "JOB default\n");
-			data = read(mainstream, mainclient);
-			#if DEBUGHYLAFAX
-				Console.WriteLine("[hylafax.job_new] \"JOB default\" data returned : {0}", data);
-			#endif
+			try {
+				write(mainstream, "JOB default\n");
+				data = read(mainstream, mainclient);
+				#if DEBUGHYLAFAX
+					Console.WriteLine("[hylafax.job_new] \"JOB default\" data returned : {0}", data);
+				#endif
 
-			write(mainstream, "JNEW\n");
-			data = read(mainstream, mainclient);
-			// data = 200 New job created: jobid: 14433 groupid: 14433.
-			tmp = data.Split(' ');
+				write(mainstream, "JNEW\n");
+				data = read(mainstream, mainclient);
+				// data = 200 New job created: jobid: 14433 groupid: 14433.
+				tmp = data.Split(' ');
 
-			#if DEBUGHYLAFAX
-				Console.WriteLine("[hylafax.job_new] \"JNEW\" data returned : {0}", data);
-			#endif
+				#if DEBUGHYLAFAX
+					Console.WriteLine("[hylafax.job_new] \"JNEW\" data returned : {0}", data);
+				#endif
+			} catch (Exception e) {
+				Console.WriteLine("Hylafax exception {0}", e);
+				return ("");
+			}
 			
 			// jobid might need to be Convert.ToInt32
 			return tmp[5];
@@ -425,7 +550,79 @@ namespace gfax {
 
 			return(s[4].TrimEnd(')','.'));
 		}
-		
+
+		private TcpClient recvfile_open (string fname)
+		{
+			TcpClient myclient = new TcpClient();
+			NetworkStream mystream;
+			IPAddress ipaddr;
+			int ipport;
+			string data;
+			
+			write(mainstream, "PASV\n");	
+			data = read(mainstream, mainclient);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.recvfile_open] data returned : {0}", data);
+			#endif
+
+			// dig out ip address and port for new connection
+			ipaddr = get_ip_addr(data);
+			ipport = get_ip_port(data);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.recvfile_open] ipaddr, port : {0}, {1}", ipaddr, ipport);
+			#endif
+			
+			try {
+				myclient.Connect(ipaddr, ipport);
+			} 
+		    catch (Exception e ) {
+				Console.WriteLine(e.ToString());
+				// handle error here
+			}
+			
+			mystream = myclient.GetStream();			
+			write(mainstream, "TYPE I\n");	
+			data = read(mainstream, mainclient);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.recvfile_open] data returned : {0}", data);
+			#endif
+
+			write(mainstream, String.Concat("RETR recvq/", fname, "\n"));
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.recvfile_open] wrote RETR");
+			#endif
+			data = read(mainstream, mainclient);
+			
+			if (data.Substring(0,3) == "550")
+				permission = false;
+			//data = returned from RETR : 550 recvq/fax000000018.tif: Operation not permitted.
+			//data = returned from RETR : 150 Opening new data connection for recvq/fax000000018.tif (2793 bytes).
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.recvfile_open] returned from RETR : {0}", data);
+			#endif
+			return myclient;
+		}
+
+		private string recvfile_close(TcpClient tc)
+		{
+			string data;
+			string[] s;
+			string fn;
+						
+			tc.Close();
+			data = read(mainstream, mainclient);
+			// data = 226 Transfer complete (FILE: /tmp/doc1101.ps).
+			// TODO error checking
+			
+			// remove white space first, Arrgggh!
+			//fn = data.Trim();
+			//s = fn.Split(' ');
+
+			//return(s[4].TrimEnd(')','.'));
+			return "";
+		}
+
+/*
 		private string getfolder (string folder)
 		{
 			TcpClient myclient = new TcpClient();
@@ -452,15 +649,83 @@ namespace gfax {
 			mystream = myclient.GetStream();			
 			/*
 			RCVFMT (receive)
-			%-7m %4p%1z %-8.8o %14.14s %7t %f
-			m - file perms
+			%4p%1z %14.14s %7t %f
 			p - number of pages
 			z - * if still receiving, <space> otherwise
-			o - file owner
 			s - sender (TIS)
 			t - time received
 			f - filename
 			
+			1= =Malaspina Univ=16Oct06=fax000000018.tif
+
+			JOBFMT (send, done)
+			j - Job identifier
+			e - Phone number
+			a - Job state (one-character symbol)
+			o - Job owner
+			P - # pages transmitted/total # pages to transmit
+			D - Total # dials/maximum # dials
+			s - Job status information from last failure
+			Y - Scheduled date and time
+			*/
+/*			// set the job format string
+			if (folder == "doneq" || folder == "sendq") {
+				jobfmt = "\"%-4j=%-14e=%1a=%-12o=%-5P=%-5D=%.35s=%-19Y\"";
+				write(mainstream, "JOBFMT "+jobfmt+"\n");
+				data = read(mainstream, mainclient);
+			}
+			if (folder == "recvq") {
+				jobfmt = "\"%4p=N%1z=%14.14s=%7t=%f\"";
+				write(mainstream, "RCVFMT "+jobfmt+"\n");
+				data = read(mainstream, mainclient);
+			}
+			write(mainstream, "LIST "+folder+"\n");
+			data = read(mystream, myclient);
+							
+						
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.getfolder] list folder data : {0}", data);
+				Console.WriteLine("[hylafax.getfolder] data length : {0}", data.Length);				
+			#endif
+
+			myclient.Close();
+			return data;
+		}
+*/
+
+		private void asyncgetfolder (string folder)
+		{
+			TcpClient myclient = new TcpClient();
+			IPAddress ipaddr;
+			int ipport;
+			string jobfmt;
+			
+			write(mainstream, "PASV\n");	
+			string data = read(mainstream, mainclient);
+			
+			// dig out ip address and port for new connection
+			ipaddr = get_ip_addr(data);
+			ipport = get_ip_port(data);
+
+			try {
+				myclient.Connect(ipaddr, ipport);
+			} 
+		    catch (Exception e ) {
+				Console.WriteLine(e.ToString());
+				// handle error here
+			}
+			
+			/*
+			RCVFMT (receive)
+			%4p%1z %14.14s %7t %f
+			p - number of pages
+			z - * if still receiving, <space> otherwise
+			s - sender (TIS)
+			t - time received
+			f - filename
+			
+			1= =Malaspina Univ=16Oct06=fax000000018.tif
+
 			JOBFMT (send, done)
 			j - Job identifier
 			e - Phone number
@@ -478,21 +743,13 @@ namespace gfax {
 				data = read(mainstream, mainclient);
 			}
 			if (folder == "recvq") {
-				jobfmt = "\"%-7m=%4p=%1z=%-8.8o=%14.14s=%7t=%f\"";
+				jobfmt = "\"%4p=N%1z=%14.14s=%7t=%f\"";
 				write(mainstream, "RCVFMT "+jobfmt+"\n");
 				data = read(mainstream, mainclient);
 			}
 			write(mainstream, "LIST "+folder+"\n");
-			data = read(mystream, myclient);
-							
-						
-			#if DEBUGHYLAFAX
-				Console.WriteLine("[hylafax.getfolder] list folder data : {0}", data);
-				Console.WriteLine("[hylafax.getfolder] data length : {0}", data.Length);				
-			#endif
-
-			myclient.Close();
-			return data;
+			//data = read(mystream, myclient);
+			asyncread(myclient, folder);
 		}
 
 		private IPAddress get_ip_addr (string ipdata)
@@ -502,7 +759,7 @@ namespace gfax {
 			string s = ipdata.Substring(index + 1, len - 1 );
 			char[] splitter  = {','};
 			string[] sa = s.Split(splitter);
-			string sipaddr = sa[0]+"."+sa[1]+"."+sa[2]+"."+sa[3];
+			string sipaddr = String.Concat(sa[0],".",sa[1],".",sa[2],".",sa[3]);
 			return IPAddress.Parse(sipaddr);
 		}
 		
@@ -529,25 +786,84 @@ namespace gfax {
 		{
 			StringBuilder buf = new StringBuilder();
 			byte[] bytes = new byte[256];
-			
+						
 			do {
 				try {
-					
 					int len = str.Read(bytes, 0, bytes.Length);
 					if (len > 0 ) {
 						buf.Append(Encoding.ASCII.GetString(bytes,0,len));
 					} 
 				} catch ( Exception e ) {
-					Console.WriteLine(e.ToString());
+					Console.WriteLine("Network IO problem " + e.ToString());
 				}
 				// Seems if we don't sleep here for a bit, we never read
 				// all the data???  Maybe a Hylafax thing.
-				System.Threading.Thread.Sleep(1);
-					
+				System.Threading.Thread.Sleep(5);
+	
 			} while (str.DataAvailable);
 			return (buf.ToString()); 
+		}			
+
+
+		//public void asyncread (TcpClient sock, string readType)
+		private void asyncread (TcpClient sock, string readType)
+		{
+			gfax.activeNetwork = true;
+			if (readType == "status") {
+				gfax.asyncServerMessageBuffer = "";
+			} else {
+				gfax.asyncFolderMessageBuffer = "";
+			}
+			
+			StateObject state = new StateObject();
+			state.client = sock;
+			state.readType = readType;
+			NetworkStream stream = sock.GetStream();
+			
+			if (stream.CanRead) {
+				try {
+					IAsyncResult ar = stream.BeginRead (state.buffer, 0, StateObject.BufferSize, 
+							new AsyncCallback(myReadCallBack), state);
+				} catch ( Exception e ) {
+					Console.WriteLine("Network IO problem " + e.ToString());
+				}
+			}
 		}
 		
+		
+		//========================================================
+		//public static void myReadCallBack(IAsyncResult ar )
+		private static void myReadCallBack(IAsyncResult ar )
+		{
+			int numberOfBytesRead;
+			StateObject state = (StateObject) ar.AsyncState;
+			NetworkStream mas = state.client.GetStream();
+			string type = null;
+			
+			numberOfBytesRead = mas.EndRead(ar);
+			state.totalBytesRead += numberOfBytesRead;
+			
+			//Console.WriteLine("Byes read ---------------> {0}", numberOfBytesRead);
+			if ( numberOfBytesRead > 0) {
+				state.messageBuffer.Append(Encoding.ASCII.GetString(state.buffer, 0, numberOfBytesRead));    
+				mas.BeginRead(state.buffer, 0, StateObject.BufferSize, 
+                        new AsyncCallback(myReadCallBack), state);  
+			} else {
+				mas.Close();
+				state.client.Close();
+				//Console.WriteLine("Byes read ---------------> {0}", state.totalBytesRead);
+				//Console.WriteLine("You received the following message : {0} " + state.messageBuffer);
+				if (state.readType == "status") {
+					gfax.asyncServerMessageBuffer = state.messageBuffer.ToString();
+				} else {
+					gfax.asyncFolderMessageBuffer = state.messageBuffer.ToString();
+				}
+				gfax.asyncReadType = state.readType;
+				mas = null;
+				state = null;
+				gfax.activeNetwork = false;
+			}
+		}
 
 		public void job_select (string jobid)
 		{
@@ -571,20 +887,44 @@ namespace gfax {
 		{
 			write (mainstream, "JSUSP\n");
 			string s = read(mainstream, mainclient);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.suspend] returned: {0}", s);
+			#endif
 		}
+		
+		// Returns
+		// 550 fax000000019.tif: Operation not permitted.
+		//
+		public int deletefile (string name)
+		{
+			write (mainstream, String.Concat("DELE ", "recvq/", name, "\n"));
+			string s = read(mainstream, mainclient);
+			if (s.Substring(0,3) == "550")
+				return(1);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.deletefile] returned: {0}", s);
+			#endif
+			return (0);
+		}
+
 
 		private void job_param (string parms)
 		{
-			write (mainstream, "JPARM " + parms+"\n");
+			write (mainstream, "JPARM " + parms + "\n");
 			string s = read(mainstream, mainclient);
 		}
 
-		private void deletefile (string name)
+		public void job_reset (string parms)
 		{
-			write (mainstream, "DELE " + name + "\n");
+			write (mainstream, "JREST " + parms + "\n");
 			string s = read(mainstream, mainclient);
+			#if DEBUGHYLAFAX
+				Console.WriteLine("[hylafax.reset] returned: {0}", s);
+			#endif
+
 		}
-		
+
+
 		private void timezone (string name)  // can be "LOCAL" or "GMT"
 		{
 			write (mainstream, "TZONE " + name + "\n");
