@@ -16,7 +16,8 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 namespace gfax {
-	using Mono.Posix;
+	using Gtk;
+	using Mono.Unix;
 	using System;
 	using System.IO;
 	using System.Collections;
@@ -34,23 +35,33 @@ namespace gfax {
 		public static bool sendWizardEmailNotify = false;
 		public static string sendWizardEmailAddress = null;
 		public static string hylafaxPassword = null;
-		public static string Procfile = null;
 		public static string ConfigDirectory = null;
 		public static string SpoolDirectory = null;
+		public static bool activeNetwork = false;
+		public static Gnome.AppBar GAppbar = null;
+		public static Gtk.ProgressBar MainProgressBar = null;
+		public static Gtk.TextBuffer GStatusTextBuffer = null;
+		public static string asyncFolderMessageBuffer = "";
+		public static string asyncServerMessageBuffer = "";
+		public static string asyncReadType = "";
+		
 		public static Efax efax;
-
+		public static Status Status;
+		public static Pulser Pulser;
+		
+				
 		const bool TOPLEVEL = true;
 				
+				
 		public static void Main (string[] args)
 		{
 		
-			//const string GCONFDIR = "/.gconf/apps/gfax";  delete me
-			
 			string HOMEDIR = Environment.GetEnvironmentVariable("HOME");
-			Procfile = HOMEDIR + "/.local/share/gfax/gfax.proc";
 			ConfigDirectory = HOMEDIR + "/.config/gfax";
 			SpoolDirectory = HOMEDIR + "/.local/share/gfax/spool";
-
+			
+									
+			// TODO put this is try/catch
 			if ( !Directory.Exists(ConfigDirectory)) {
 				if ( !Directory.Exists(HOMEDIR + "/.config")) {
 					Directory.CreateDirectory(HOMEDIR + "/.config");
@@ -76,22 +87,19 @@ namespace gfax {
 				Directory.CreateDirectory(HOMEDIR + "/.local/share/gfax/spool/recq");
 			}
 			
-					
+			// Clean out the spool/tif directory - incoming faxes
+			if ( !Directory.Exists(HOMEDIR + "/.local/share/gfax/spool/tif")) {
+				Directory.CreateDirectory(HOMEDIR + "/.local/share/gfax/spool/tif");
+			} else {
+				Directory.Delete(HOMEDIR + "/.local/share/gfax/spool/tif/", true);
+				Directory.CreateDirectory(HOMEDIR + "/.local/share/gfax/spool/tif");
+			}
+			
 			string filename = null;	
 			
 			// Initialize GETTEXT
 			Catalog.Init ("gfax", Defines.GNOME_LOCALE_DIR);
 
-			// Touch the gfax.proc file, we want it to exist
-			try {
-				StreamWriter sw = File.CreateText(Procfile);
-				sw.Close();
-			} catch (Exception e) {
-				Console.WriteLine(Catalog.GetString("Can't create .gfax.proc"));
-				Environment.Exit(1);
-			};
-
-			
 			// handle command line args ourselves					
 			for (int i=0; i < args.Length; i++ ) {
 				//Console.WriteLine("{0} {1}", i, args[i]);
@@ -128,6 +136,7 @@ namespace gfax {
 If you are running Gfax immediately 
 after an installation, you may have 
 to log out and log back in again."), TOPLEVEL);
+				Console.WriteLine("Exception in main.cs {0}", e);
 				Environment.Exit(0);
 			}
 
@@ -141,7 +150,6 @@ to log out and log back in again."), TOPLEVEL);
 					
 					// Start the fax daemon if efax
 					if (Settings.TransmitAgent == "efax") {
-						
 						efax = new Efax();
 						efax.run_efaxd();
 					}
