@@ -16,7 +16,7 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 namespace gfax {
-	using Mono.Posix;
+	using Mono.Unix;
 	using System;
 	using System.IO;
 	using System.Text;
@@ -57,9 +57,11 @@ namespace gfax {
 			#endif
 			
 			int numberOfBooks = get_number_of_books();
+			pbooks = new Phonebook[numberOfBooks];			
 			
-			//TODO check for empty file (numberOfBooks > 0)
-			pbooks = new Phonebook[numberOfBooks];
+			if (numberOfBooks == 0) {
+				return (pbooks);
+			}
 			
 				
 			try { 
@@ -94,6 +96,7 @@ namespace gfax {
 				return pbooks;			
 			}
 			catch (Exception e) { 
+				Console.WriteLine("get_phonebooks - Exception in phonebook.cs {0}", e);
 				return pbooks; 
 			}
 
@@ -107,8 +110,12 @@ namespace gfax {
 			// TODO  get location from gconf
 			string PHONEBOOKS = gfax.ConfigDirectory + "/phonebooks";
 			
-			try { outfile = File.CreateText(PHONEBOOKS); }
-			catch (Exception e) { return; }
+			try { 
+				outfile = File.CreateText(PHONEBOOKS); 
+			} catch (Exception e) { 
+				Console.WriteLine("Exception in phonebook.cs {0}", e);
+				return; 
+			}
 
 			outfile.WriteLine("<gfax>");
 			Console.WriteLine("Len :{0}", pbooks.Length);
@@ -167,6 +174,7 @@ namespace gfax {
 						File.Delete(deleteme);
 			}
 			catch (Exception e) {
+				Console.WriteLine("delete_book - Exception in phonebook.cs {0}", e);
 				return; 
 			}
 		}
@@ -216,6 +224,7 @@ namespace gfax {
 			}
 			catch (Exception e) { 
 				// TODO catch file ops error
+				Console.WriteLine("add_book - Exception in phonebook.cs {0}", e);
 				return;
 			}
 
@@ -235,7 +244,10 @@ namespace gfax {
 			if (p.Type == "gfax") {
 			// TODO error reporting
 				try { outfile = File.CreateText(p.Path); }
-				catch (Exception e) { return; }
+				catch (Exception e) {
+					Console.WriteLine("save_phonebook_items - Exception in phonebook.cs {0}", e);
+					return; 
+				}
 			
 				outfile.WriteLine("#Gfax phone book");
 			
@@ -253,10 +265,15 @@ namespace gfax {
 		{
 			StreamReader infile = null;
 			
+			// If it doesn't exist yet just return
+			if (!File.Exists(p.Path))
+				return(null);
+				
 			try { 
 				infile = File.OpenText(p.Path);
 			}
 			catch (Exception e) { 
+				Console.WriteLine("open_phonebook - Exception in phonebook.cs {0}", e);
 				return null;
 			}
 			return infile;
@@ -331,9 +348,13 @@ namespace gfax {
 			// TODO  get location from gconf
 			string PHONEBOOKS = gfax.ConfigDirectory + "/phonebooks";
 			
+			if (!File.Exists(PHONEBOOKS))
+				return(0);
+				
 			try { 
 				infile = File.OpenText(PHONEBOOKS); 
 			} catch (Exception e) { 
+				Console.WriteLine("get_number_of_books - Exception in phonebook.cs {0}", e);
 				return 0; 
 			}
 			
