@@ -16,7 +16,7 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 namespace gfax {
-	using Mono.Posix;
+	using Mono.Unix;
 	using System;
 	using System.IO;
 	using GLib;
@@ -141,7 +141,7 @@ namespace gfax {
 		}
 
 		// If we double click the phonebook.
-		private void on_book_treeview_row_activated(object o, EventArgs args)
+		private void on_book_treeview_row_activated(object o, RowActivatedArgs args)
 		{	
 			/*
 			ArrayList bsdest = new ArrayList();
@@ -167,7 +167,7 @@ namespace gfax {
 		}
 
 
-		private void on_list_treeview_row_activated(object o, EventArgs args)
+		private void on_list_treeview_row_activated(object o, RowActivatedArgs args)
 		{
 		}
 
